@@ -21,7 +21,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import collections
-import copy
 import json
 import logging
 import os
@@ -41,31 +40,35 @@ from . import metadata
 from . import net
 from . import signindex
 from fdroidserver.common import FDroidPopen, FDroidPopenBytes, load_stats_fdroid_signing_key_fingerprints
-from fdroidserver.exception import FDroidException, VerificationException, MetaDataException
+from fdroidserver.exception import FDroidException, VerificationException
 
 
-def make(apps, sortedids, apks, repodir, archive):
+def make(apps, apks, repodir, archive):
     """Generate the repo index files.
 
     This requires properly initialized options and config objects.
 
-    :param apps: fully populated apps list
-    :param sortedids: app package IDs, sorted
-    :param apks: full populated apks list
+    :param apps: OrderedDict of apps to go into the index, each app should have
+                 at least one associated apk
+    :param apks: list of apks to go into the index
     :param repodir: the repo directory
     :param archive: True if this is the archive repo, False if it's the
                     main one.
     """
     from fdroidserver.update import METADATA_VERSION
 
-    def _resolve_description_link(appid):
-        if appid in apps:
-            return "fdroid.app:" + appid, apps[appid].Name
-        raise MetaDataException("Cannot resolve app id " + appid)
-
-    if not common.options.nosign:
+    if hasattr(common.options, 'nosign') and common.options.nosign:
+        if 'keystore' not in common.config and 'repo_pubkey' not in common.config:
+            raise FDroidException(_('"repo_pubkey" must be present in config.yml when using --nosign!'))
+    else:
         common.assert_config_keystore(common.config)
 
+    # Historically the index has been sorted by App Name, so we enforce this ordering here
+    sortedids = sorted(apps, key=lambda appid: common.get_app_display_name(apps[appid]).upper())
+    sortedapps = collections.OrderedDict()
+    for appid in sortedids:
+        sortedapps[appid] = apps[appid]
+
     repodict = collections.OrderedDict()
     repodict['timestamp'] = datetime.utcnow().replace(tzinfo=timezone.utc)
     repodict['version'] = METADATA_VERSION
@@ -75,20 +78,21 @@ def make(apps, sortedids, apks, repodir, archive):
 
     if archive:
         repodict['name'] = common.config['archive_name']
-        repodict['icon'] = os.path.basename(common.config['archive_icon'])
-        repodict['address'] = common.config['archive_url']
+        repodict['icon'] = common.config.get('archive_icon', common.default_config['repo_icon'])
         repodict['description'] = common.config['archive_description']
-        urlbasepath = os.path.basename(urllib.parse.urlparse(common.config['archive_url']).path)
+        archive_url = common.config.get('archive_url', common.config['repo_url'][:-4] + 'archive')
+        repodict['address'] = archive_url
+        urlbasepath = os.path.basename(urllib.parse.urlparse(archive_url).path)
     else:
         repodict['name'] = common.config['repo_name']
-        repodict['icon'] = os.path.basename(common.config['repo_icon'])
+        repodict['icon'] = common.config.get('repo_icon', common.default_config['repo_icon'])
         repodict['address'] = common.config['repo_url']
         repodict['description'] = common.config['repo_description']
         urlbasepath = os.path.basename(urllib.parse.urlparse(common.config['repo_url']).path)
 
     mirrorcheckfailed = False
     mirrors = []
-    for mirror in sorted(common.config.get('mirrors', [])):
+    for mirror in common.config.get('mirrors', []):
         base = os.path.basename(urllib.parse.urlparse(mirror).path.rstrip('/'))
         if common.config.get('nonstandardwebroot') is not True and base != 'fdroid':
             logging.error(_("mirror '%s' does not end with 'fdroid'!") % mirror)
@@ -106,21 +110,6 @@ def make(apps, sortedids, apks, repodir, archive):
     if mirrors:
         repodict['mirrors'] = mirrors
 
-    appsWithPackages = collections.OrderedDict()
-    for packageName in sortedids:
-        app = apps[packageName]
-        if app['Disabled']:
-            continue
-
-        # only include apps with packages
-        for apk in apks:
-            if apk['packageName'] == packageName:
-                newapp = copy.copy(app)  # update wiki needs unmodified description
-                newapp['Description'] = metadata.description_html(app['Description'],
-                                                                  _resolve_description_link)
-                appsWithPackages[packageName] = newapp
-                break
-
     requestsdict = collections.OrderedDict()
     for command in ('install', 'uninstall'):
         packageNames = []
@@ -136,9 +125,9 @@ def make(apps, sortedids, apks, repodir, archive):
 
     fdroid_signing_key_fingerprints = load_stats_fdroid_signing_key_fingerprints()
 
-    make_v0(appsWithPackages, apks, repodir, repodict, requestsdict,
+    make_v0(sortedapps, apks, repodir, repodict, requestsdict,
             fdroid_signing_key_fingerprints)
-    make_v1(appsWithPackages, apks, repodir, repodict, requestsdict,
+    make_v1(sortedapps, apks, repodir, repodict, requestsdict,
             fdroid_signing_key_fingerprints)
 
 
@@ -149,7 +138,7 @@ def make_v1(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
             return sorted(list(obj))
         if isinstance(obj, datetime):
             # Java prefers milliseconds
-            # we also need to accound for time zone/daylight saving time
+            # we also need to account for time zone/daylight saving time
             return int(calendar.timegm(obj.timetuple()) * 1000)
         if isinstance(obj, dict):
             d = collections.OrderedDict()
@@ -173,8 +162,8 @@ def make_v1(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
         for k, v in sorted(appdict.items()):
             if not v:
                 continue
-            if k in ('builds', 'comments', 'metadatapath',
-                     'ArchivePolicy', 'AutoUpdateMode', 'MaintainerNotes',
+            if k in ('Builds', 'comments', 'metadatapath',
+                     'ArchivePolicy', 'AutoName', 'AutoUpdateMode', 'MaintainerNotes',
                      'Provides', 'Repo', 'RepoType', 'RequiresRoot',
                      'UpdateCheckData', 'UpdateCheckIgnore', 'UpdateCheckMode',
                      'UpdateCheckName', 'NoSourceSince', 'VercodeOperation'):
@@ -187,10 +176,6 @@ def make_v1(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
                 k = 'suggestedVersionCode'
             elif k == 'CurrentVersion':  # TODO make SuggestedVersionName the canonical name
                 k = 'suggestedVersionName'
-            elif k == 'AutoName':
-                if 'Name' not in apps[packageName]:
-                    d['name'] = v
-                continue
             else:
                 k = k[:1].lower() + k[1:]
             d[k] = v
@@ -216,7 +201,7 @@ def make_v1(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
         if not package.get('versionName'):
             app = apps[packageName]
             versionCodeStr = str(package['versionCode'])  # TODO build.versionCode should be int!
-            for build in app['builds']:
+            for build in app.get('Builds', []):
                 if build['versionCode'] == versionCodeStr:
                     versionName = build.get('versionName')
                     logging.info(_('Overriding blank versionName in {apkfilename} from metadata: {version}')
@@ -311,11 +296,6 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
         value = str(apk[key])
         addElement(name, value, doc, parent)
 
-    def addElementCDATA(name, value, doc, parent):
-        el = doc.createElement(name)
-        el.appendChild(doc.createCDATASection(value))
-        parent.appendChild(el)
-
     def addElementCheckLocalized(name, app, key, doc, parent, default=''):
         """Fill in field from metadata or localized block
 
@@ -346,6 +326,8 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
             value = localized[lang].get(lkey)
         if not value:
             value = default
+        if not value and name == 'name' and app.get('AutoName'):
+            value = app['AutoName']
         el.appendChild(doc.createTextNode(value))
         parent.appendChild(el)
 
@@ -353,7 +335,7 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
     doc.appendChild(root)
 
     repoel = doc.createElement("repo")
-    repoel.setAttribute("icon", os.path.basename(repodict['icon']))
+    repoel.setAttribute("icon", repodict['icon'])
     if 'maxage' in repodict:
         repoel.setAttribute("maxage", str(repodict['maxage']))
     repoel.setAttribute("name", repodict['name'])
@@ -378,15 +360,18 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
     for appid, appdict in apps.items():
         app = metadata.App(appdict)
 
-        if app.Disabled is not None:
+        if app.get('Disabled') is not None:
             continue
 
         # Get a list of the apks for this app...
         apklist = []
+        name_from_apk = None
         apksbyversion = collections.defaultdict(lambda: [])
         for apk in apks:
             if apk.get('versionCode') and apk.get('packageName') == appid:
                 apksbyversion[apk['versionCode']].append(apk)
+                if name_from_apk is None:
+                    name_from_apk = apk.get('name')
         for versionCode, apksforver in apksbyversion.items():
             fdroidsig = fdroid_signing_key_fingerprints.get(appid, {}).get('signer')
             fdroid_signed_apk = None
@@ -418,14 +403,14 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
         if app.lastUpdated:
             addElement('lastupdated', app.lastUpdated.strftime('%Y-%m-%d'), doc, apel)
 
-        addElementCheckLocalized('name', app, 'Name', doc, apel)
+        addElementCheckLocalized('name', app, 'Name', doc, apel, name_from_apk)
         addElementCheckLocalized('summary', app, 'Summary', doc, apel)
 
         if app.icon:
             addElement('icon', app.icon, doc, apel)
 
         addElementCheckLocalized('desc', app, 'Description', doc, apel,
-                                 '<p>No description available</p>')
+                                 'No description available')
 
         addElement('license', app.License, doc, apel)
         if app.Categories:
@@ -459,7 +444,7 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
         if app.RequiresRoot:
             addElement('requirements', 'root', doc, apel)
 
-        # Sort the apk list into version order, just so the web site
+        # Sort the APK list into version order, just so the web site
         # doesn't have to do any work by default...
         apklist = sorted(apklist, key=lambda apk: apk['versionCode'], reverse=True)
 
@@ -486,10 +471,10 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
         for apk in apklist:
             file_extension = common.get_file_extension(apk['apkName'])
             # find the APK for the "Current Version"
-            if current_version_code < apk['versionCode']:
-                current_version_code = apk['versionCode']
             if current_version_code < int(app.CurrentVersionCode):
                 current_version_file = apk['apkName']
+            if current_version_code < apk['versionCode']:
+                current_version_code = apk['versionCode']
 
             apkel = doc.createElement("package")
             apel.appendChild(apkel)
@@ -497,7 +482,7 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
             versionName = apk.get('versionName')
             if not versionName:
                 versionCodeStr = str(apk['versionCode'])  # TODO build.versionCode should be int!
-                for build in app.builds:
+                for build in app.get('Builds', []):
                     if build['versionCode'] == versionCodeStr and 'versionName' in build:
                         versionName = build['versionName']
                         break
@@ -563,7 +548,12 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
                 and common.config['make_current_version_link'] \
                 and repodir == 'repo':  # only create these
             namefield = common.config['current_version_name_source']
-            sanitized_name = re.sub(b'''[ '"&%?+=/]''', b'', app.get(namefield).encode('utf-8'))
+            name = app.get(namefield)
+            if not name and namefield == 'Name':
+                name = app.get('localized', {}).get('en-US', {}).get('name')
+            if not name:
+                name = app.id
+            sanitized_name = re.sub(b'''[ '"&%?+=/]''', b'', name.encode('utf-8'))
             apklinkname = sanitized_name + os.path.splitext(current_version_file)[1].encode('utf-8')
             current_version_path = os.path.join(repodir, current_version_file).encode('utf-8', 'surrogateescape')
             if os.path.islink(apklinkname):
@@ -586,7 +576,8 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
     with open(os.path.join(repodir, 'index.xml'), 'wb') as f:
         f.write(output)
 
-    if 'repo_keyalias' in common.config:
+    if 'repo_keyalias' in common.config \
+       or (common.options.nosign and 'repo_pubkey' in common.config):
 
         if common.options.nosign:
             logging.info(_("Creating unsigned index in preparation for signing"))
@@ -612,8 +603,21 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
 
     # Copy the repo icon into the repo directory...
     icon_dir = os.path.join(repodir, 'icons')
-    iconfilename = os.path.join(icon_dir, os.path.basename(common.config['repo_icon']))
-    shutil.copyfile(common.config['repo_icon'], iconfilename)
+    repo_icon = common.config.get('repo_icon', common.default_config['repo_icon'])
+    iconfilename = os.path.join(icon_dir, os.path.basename(repo_icon))
+    if os.path.exists(repo_icon):
+        shutil.copyfile(common.config['repo_icon'], iconfilename)
+    else:
+        logging.warning(_('repo_icon "repo/icons/%s" does not exist, generating placeholder.')
+                        % repo_icon)
+        os.makedirs(os.path.dirname(iconfilename), exist_ok=True)
+        try:
+            import qrcode
+            qrcode.make(common.config['repo_url']).save(iconfilename)
+        except Exception:
+            exampleicon = os.path.join(common.get_examples_dir(),
+                                       common.default_config['repo_icon'])
+            shutil.copy(exampleicon, iconfilename)
 
 
 def extract_pubkey():
@@ -630,7 +634,7 @@ def extract_pubkey():
                               '-alias', common.config['repo_keyalias'],
                               '-keystore', common.config['keystore'],
                               '-storepass:env', 'FDROID_KEY_STORE_PASS']
-                             + common.config['smartcardoptions'],
+                             + list(common.config['smartcardoptions']),
                              envs=env_vars, output=False, stderr_to_stdout=False)
         if p.returncode != 0 or len(p.output) < 20:
             msg = "Failed to get repo pubkey!"
@@ -654,7 +658,7 @@ def get_mirror_service_urls(url):
     '''
 
     if url.startswith('git@'):
-        url = re.sub(r'^git@(.*):(.*)', r'https://\1/\2', url)
+        url = re.sub(r'^git@([^:]+):(.+)', r'https://\1/\2', url)
 
     segments = url.split("/")
 
@@ -682,10 +686,9 @@ def get_mirror_service_urls(url):
         # Gitlab-like Pages segments "https://user.gitlab.io/repo/folder"
         gitlab_pages = ["https:", "", user + ".gitlab.io", repo, folder]
         urls.append('/'.join(gitlab_pages))
-        # Gitlab Raw "https://gitlab.com/user/repo/raw/branch/folder"
-        gitlab_raw = segments + ['raw', branch, folder]
+        # GitLab Raw "https://gitlab.com/user/repo/-/raw/branch/folder"
+        gitlab_raw = segments + ['-', 'raw', branch, folder]
         urls.append('/'.join(gitlab_raw))
-        return urls
 
     return urls
 
@@ -732,7 +735,11 @@ def download_repo_index(url_str, etag=None, verify_fingerprint=True, timeout=600
 def get_index_from_jar(jarfile, fingerprint=None):
     """Returns the data, public key, and fingerprint from index-v1.jar
 
+    :param fingerprint is the SHA-256 fingerprint of signing key. Only
+           hex digits count, all other chars will can be discarded.
+
     :raises: VerificationException() if the repository could not be verified
+
     """
 
     logging.debug(_('Verifying index signature:'))
@@ -740,7 +747,8 @@ def get_index_from_jar(jarfile, fingerprint=None):
     with zipfile.ZipFile(jarfile) as jar:
         public_key, public_key_fingerprint = get_public_key_from_jar(jar)
         if fingerprint is not None:
-            if fingerprint.upper() != public_key_fingerprint:
+            fingerprint = re.sub(r'[^0-9A-F]', r'', fingerprint.upper())
+            if fingerprint != public_key_fingerprint:
                 raise VerificationException(_("The repository's fingerprint does not match."))
         data = json.loads(jar.read('index-v1.json').decode())
         return data, public_key, public_key_fingerprint
