@@ -33,7 +33,7 @@ options = None
 
 
 def enforce_https(domain):
-    return (re.compile(r'^[^h][^t][^t][^p][^s]://[^/]*' + re.escape(domain) + r'(/.*)?', re.IGNORECASE),
+    return (re.compile(r'^http://([^/]*\.)?' + re.escape(domain) + r'(/.*)?', re.IGNORECASE),
             domain + " URLs should always use https://")
 
 
@@ -120,9 +120,11 @@ http_url_shorteners = [
 ]
 
 http_checks = https_enforcings + http_url_shorteners + [
-    (re.compile(r'.*github\.com/[^/]+/[^/]+\.git'),
+    (re.compile(r'^(?!https?://)[^/]+'),
+     _("URL must start with https:// or http://")),
+    (re.compile(r'^https://(github|gitlab)\.com(/[^/]+){2,3}\.git'),
      _("Appending .git is not necessary")),
-    (re.compile(r'.*://[^/]*(github|gitlab|bitbucket|rawgit)[^/]*/([^/]+/){1,3}master'),
+    (re.compile(r'^https://[^/]*(github|gitlab|bitbucket|rawgit|githubusercontent)\.[a-zA-Z]+/([^/]+/){2,3}master/'),
      _("Use /HEAD instead of /master to point at a file in the default branch")),
 ]
 
@@ -189,9 +191,9 @@ def check_regexes(app):
             v = app.get(f)
             t = metadata.fieldtype(f)
             if t == metadata.TYPE_MULTILINE:
-                for l in v.splitlines():
-                    if m.match(l):
-                        yield "%s at line '%s': %s" % (f, l, r)
+                for line in v.splitlines():
+                    if m.match(line):
+                        yield "%s at line '%s': %s" % (f, line, r)
             else:
                 if v is None:
                     continue
@@ -232,7 +234,7 @@ def check_vercode_operation(app):
 
 
 def check_ucm_tags(app):
-    lastbuild = get_lastbuild(app.builds)
+    lastbuild = get_lastbuild(app.get('Builds', []))
     if (lastbuild is not None
             and lastbuild.commit
             and app.UpdateCheckMode == 'RepoManifest'
@@ -240,7 +242,7 @@ def check_ucm_tags(app):
             and lastbuild.versionCode == app.CurrentVersionCode
             and not lastbuild.forcevercode
             and any(s in lastbuild.commit for s in '.,_-/')):
-        yield _("Last used commit '{commit}' looks like a tag, but Update Check Mode is '{ucm}'")\
+        yield _("Last used commit '{commit}' looks like a tag, but UpdateCheckMode is '{ucm}'")\
             .format(commit=lastbuild.commit, ucm=app.UpdateCheckMode)
 
 
@@ -276,7 +278,7 @@ def check_old_links(app):
 
 def check_useless_fields(app):
     if app.UpdateCheckName == app.id:
-        yield _("Update Check Name is set to the known app id - it can be removed")
+        yield _("UpdateCheckName is set to the known application ID, it can be removed")
 
 
 filling_ucms = re.compile(r'^(Tags.*|RepoManifest.*)')
@@ -285,7 +287,7 @@ filling_ucms = re.compile(r'^(Tags.*|RepoManifest.*)')
 def check_checkupdates_ran(app):
     if filling_ucms.match(app.UpdateCheckMode):
         if not app.AutoName and not app.CurrentVersion and app.CurrentVersionCode == '0':
-            yield _("UCM is set but it looks like checkupdates hasn't been run yet")
+            yield _("UpdateCheckMode is set but it looks like checkupdates hasn't been run yet")
 
 
 def check_empty_fields(app):
@@ -321,9 +323,6 @@ def check_categories(app):
 
 
 def check_duplicates(app):
-    if app.Name and app.Name == app.AutoName:
-        yield _("Name '%s' is just the auto name - remove it") % app.Name
-
     links_seen = set()
     for f in ['Source Code', 'Web Site', 'Issue Tracker', 'Changelog']:
         v = app.get(f)
@@ -335,7 +334,7 @@ def check_duplicates(app):
         else:
             links_seen.add(v)
 
-    name = app.Name or app.AutoName
+    name = common.get_app_display_name(app)
     if app.Summary and name:
         if app.Summary.lower() == name.lower():
             yield _("Summary '%s' is just the app's name") % app.Summary
@@ -345,12 +344,12 @@ def check_duplicates(app):
             yield _("Description '%s' is just the app's summary") % app.Summary
 
     seenlines = set()
-    for l in app.Description.splitlines():
-        if len(l) < 1:
+    for line in app.Description.splitlines():
+        if len(line) < 1:
             continue
-        if l in seenlines:
+        if line in seenlines:
             yield _("Description has a duplicate line")
-        seenlines.add(l)
+        seenlines.add(line)
 
 
 desc_url = re.compile(r'(^|[^[])\[([^ ]+)( |\]|$)')
@@ -369,25 +368,25 @@ def check_bulleted_lists(app):
     validchars = ['*', '#']
     lchar = ''
     lcount = 0
-    for l in app.Description.splitlines():
-        if len(l) < 1:
+    for line in app.Description.splitlines():
+        if len(line) < 1:
             lcount = 0
             continue
 
-        if l[0] == lchar and l[1] == ' ':
+        if line[0] == lchar and line[1] == ' ':
             lcount += 1
             if lcount > 2 and lchar not in validchars:
                 yield _("Description has a list (%s) but it isn't bulleted (*) nor numbered (#)") % lchar
                 break
         else:
-            lchar = l[0]
+            lchar = line[0]
             lcount = 1
 
 
 def check_builds(app):
     supported_flags = set(metadata.build_flags)
     # needed for YAML and JSON
-    for build in app.builds:
+    for build in app.get('Builds', []):
         if build.disable:
             if build.disable.startswith('Generated by import.py'):
                 yield _("Build generated by `fdroid import` - remove disable line once ready")
@@ -422,7 +421,7 @@ def check_files_dir(app):
         files.add(name)
 
     used = {'signatures', }
-    for build in app.builds:
+    for build in app.get('Builds', []):
         for fname in build.patch:
             if fname not in files:
                 yield _("Unknown file '{filename}' in build '{versionName}'")\
@@ -442,10 +441,17 @@ def check_format(app):
 
 
 def check_license_tag(app):
-    '''Ensure all license tags are in https://spdx.org/license-list'''
-    if app.License.rstrip('+') not in SPDX:
-        yield _('Invalid license tag "%s"! Use only tags from https://spdx.org/license-list') \
-            % (app.License)
+    '''Ensure all license tags contain only valid/approved values'''
+    if config['lint_licenses'] is None:
+        return
+    if app.License not in config['lint_licenses']:
+        if config['lint_licenses'] == APPROVED_LICENSES:
+            yield _('Unexpected license tag "{}"! Only use FSF or OSI '
+                    'approved tags from https://spdx.org/license-list') \
+                .format(app.License)
+        else:
+            yield _('Unexpected license tag "{}"! Only use license tags '
+                    'configured in your config file').format(app.License)
 
 
 def check_extlib_dir(apps):
@@ -457,7 +463,7 @@ def check_extlib_dir(apps):
 
     used = set()
     for app in apps:
-        for build in app.builds:
+        for build in app.get('Builds', []):
             for path in build.extlibs:
                 if path not in unused_extlib_files:
                     yield _("{appid}: Unknown extlib {path} in build '{versionName}'")\
@@ -485,7 +491,7 @@ def check_app_field_types(app):
         t = metadata.fieldtype(field)
         if v is None:
             continue
-        elif field == 'builds':
+        elif field == 'Builds':
             if not isinstance(v, list):
                 yield(_("{appid}: {field} must be a '{type}', but it is a '{fieldtype}'!")
                       .format(appid=app.id, field=field,
@@ -506,24 +512,20 @@ def check_for_unsupported_metadata_files(basedir=""):
     global config
 
     return_value = False
-    formats = config['accepted_formats']
     for f in glob.glob(basedir + 'metadata/*') + glob.glob(basedir + 'metadata/.*'):
         if os.path.isdir(f):
-            exists = False
-            for t in formats:
-                exists = exists or os.path.exists(f + '.' + t)
-            if not exists:
+            if not os.path.exists(f + '.yml'):
                 print(_('"%s/" has no matching metadata file!') % f)
                 return_value = True
-        elif os.path.splitext(f)[1][1:] in formats:
+        elif f.endswith('.yml'):
             packageName = os.path.splitext(os.path.basename(f))[0]
             if not common.is_valid_package_name(packageName):
                 print('"' + packageName + '" is an invalid package name!\n'
                       + 'https://developer.android.com/studio/build/application-id')
                 return_value = True
         else:
-            print('"' + f.replace(basedir, '')
-                  + '" is not a supported file format: (' + ','.join(formats) + ')')
+            print(_('"{path}" is not a supported file format (use: metadata/*.yml)')
+                  .format(path=f.replace(basedir, '')))
             return_value = True
 
     return return_value
@@ -539,7 +541,7 @@ def check_current_version_code(app):
     if cv is not None and int(cv) == 0:
         return
 
-    builds = app.get('builds')
+    builds = app.get('Builds')
     active_builds = 0
     min_versionCode = None
     if builds:
@@ -563,11 +565,14 @@ def main():
     global config, options
 
     # Parse command line...
-    parser = ArgumentParser(usage="%(prog)s [options] [APPID [APPID ...]]")
+    parser = ArgumentParser()
     common.setup_global_opts(parser)
     parser.add_argument("-f", "--format", action="store_true", default=False,
                         help=_("Also warn about formatting issues, like rewritemeta -l"))
-    parser.add_argument("appid", nargs='*', help=_("applicationId in the form APPID"))
+    parser.add_argument('--force-yamllint', action="store_true", default=False,
+                        help=_("When linting the entire repository yamllint is disabled by default. "
+                               "This option forces yamllint regardless."))
+    parser.add_argument("appid", nargs='*', help=_("application ID of file to operate on"))
     metadata.add_metadata_arguments(parser)
     options = parser.parse_args()
     metadata.warnings_action = options.W
@@ -575,7 +580,7 @@ def main():
     config = common.read_config(options)
 
     # Get all apps...
-    allapps = metadata.read_metadata(xref=True)
+    allapps = metadata.read_metadata(options.appid)
     apps = common.read_app_args(options.appid, allapps, False)
 
     anywarns = check_for_unsupported_metadata_files()
@@ -593,6 +598,33 @@ def main():
         if app.Disabled:
             continue
 
+        if options.force_yamllint:
+            import yamllint  # throw error if it is not installed
+            yamllint  # make pyflakes ignore this
+
+        # only run yamllint when linting individual apps.
+        if len(options.appid) > 0 or options.force_yamllint:
+
+            # run yamllint on app metadata
+            ymlpath = os.path.join('metadata', appid + '.yml')
+            if os.path.isfile(ymlpath):
+                yamllintresult = common.run_yamllint(ymlpath)
+                if yamllintresult != '':
+                    print(yamllintresult)
+
+            # run yamllint on srclib metadata
+            srclibs = set()
+            for build in app.get('Builds', []):
+                for srclib in build.srclibs:
+                    srclibs.add(srclib)
+            for srclib in srclibs:
+                name, ref, number, subdir = common.parse_srclib_spec(srclib)
+                srclibpath = os.path.join('srclibs', name + '.yml')
+                if os.path.isfile(srclibpath):
+                    yamllintresult = common.run_yamllint(srclibpath)
+                    if yamllintresult != '':
+                        print(yamllintresult)
+
         app_check_funcs = [
             check_app_field_types,
             check_regexes,
@@ -624,356 +656,187 @@ def main():
         sys.exit(1)
 
 
-# A compiled, public domain list of official SPDX license tags from:
-# https://github.com/sindresorhus/spdx-license-list/blob/v4.0.0/spdx-simple.json
-# The deprecated license tags have been removed from the list, they are at the
-# bottom, starting after the last license tags that start with Z.
-# This is at the bottom, since its a long list of data
-SPDX = [
-    "PublicDomain",  # an F-Droid addition, until we can enforce a better option
-    "0BSD",
-    "AAL",
-    "Abstyles",
-    "Adobe-2006",
-    "Adobe-Glyph",
-    "ADSL",
-    "AFL-1.1",
-    "AFL-1.2",
-    "AFL-2.0",
-    "AFL-2.1",
-    "AFL-3.0",
-    "Afmparse",
-    "AGPL-1.0",
-    "AGPL-3.0-only",
-    "AGPL-3.0-or-later",
-    "Aladdin",
-    "AMDPLPA",
-    "AML",
-    "AMPAS",
-    "ANTLR-PD",
-    "Apache-1.0",
-    "Apache-1.1",
-    "Apache-2.0",
-    "APAFML",
-    "APL-1.0",
-    "APSL-1.0",
-    "APSL-1.1",
-    "APSL-1.2",
-    "APSL-2.0",
-    "Artistic-1.0-cl8",
-    "Artistic-1.0-Perl",
-    "Artistic-1.0",
-    "Artistic-2.0",
-    "Bahyph",
-    "Barr",
-    "Beerware",
-    "BitTorrent-1.0",
-    "BitTorrent-1.1",
-    "Borceux",
-    "BSD-1-Clause",
-    "BSD-2-Clause-FreeBSD",
-    "BSD-2-Clause-NetBSD",
-    "BSD-2-Clause-Patent",
-    "BSD-2-Clause",
-    "BSD-3-Clause-Attribution",
-    "BSD-3-Clause-Clear",
-    "BSD-3-Clause-LBNL",
-    "BSD-3-Clause-No-Nuclear-License-2014",
-    "BSD-3-Clause-No-Nuclear-License",
-    "BSD-3-Clause-No-Nuclear-Warranty",
-    "BSD-3-Clause",
-    "BSD-4-Clause-UC",
-    "BSD-4-Clause",
-    "BSD-Protection",
-    "BSD-Source-Code",
-    "BSL-1.0",
-    "bzip2-1.0.5",
-    "bzip2-1.0.6",
-    "Caldera",
-    "CATOSL-1.1",
-    "CC-BY-1.0",
-    "CC-BY-2.0",
-    "CC-BY-2.5",
-    "CC-BY-3.0",
-    "CC-BY-4.0",
-    "CC-BY-NC-1.0",
-    "CC-BY-NC-2.0",
-    "CC-BY-NC-2.5",
-    "CC-BY-NC-3.0",
-    "CC-BY-NC-4.0",
-    "CC-BY-NC-ND-1.0",
-    "CC-BY-NC-ND-2.0",
-    "CC-BY-NC-ND-2.5",
-    "CC-BY-NC-ND-3.0",
-    "CC-BY-NC-ND-4.0",
-    "CC-BY-NC-SA-1.0",
-    "CC-BY-NC-SA-2.0",
-    "CC-BY-NC-SA-2.5",
-    "CC-BY-NC-SA-3.0",
-    "CC-BY-NC-SA-4.0",
-    "CC-BY-ND-1.0",
-    "CC-BY-ND-2.0",
-    "CC-BY-ND-2.5",
-    "CC-BY-ND-3.0",
-    "CC-BY-ND-4.0",
-    "CC-BY-SA-1.0",
-    "CC-BY-SA-2.0",
-    "CC-BY-SA-2.5",
-    "CC-BY-SA-3.0",
-    "CC-BY-SA-4.0",
-    "CC0-1.0",
-    "CDDL-1.0",
-    "CDDL-1.1",
-    "CDLA-Permissive-1.0",
-    "CDLA-Sharing-1.0",
-    "CECILL-1.0",
-    "CECILL-1.1",
-    "CECILL-2.0",
-    "CECILL-2.1",
-    "CECILL-B",
-    "CECILL-C",
-    "ClArtistic",
-    "CNRI-Jython",
-    "CNRI-Python-GPL-Compatible",
-    "CNRI-Python",
-    "Condor-1.1",
-    "CPAL-1.0",
-    "CPL-1.0",
-    "CPOL-1.02",
-    "Crossword",
-    "CrystalStacker",
-    "CUA-OPL-1.0",
-    "Cube",
-    "curl",
-    "D-FSL-1.0",
-    "diffmark",
-    "DOC",
-    "Dotseqn",
-    "DSDP",
-    "dvipdfm",
-    "ECL-1.0",
-    "ECL-2.0",
-    "EFL-1.0",
-    "EFL-2.0",
-    "eGenix",
-    "Entessa",
-    "EPL-1.0",
-    "EPL-2.0",
-    "ErlPL-1.1",
-    "EUDatagrid",
-    "EUPL-1.0",
-    "EUPL-1.1",
-    "EUPL-1.2",
-    "Eurosym",
-    "Fair",
-    "Frameworx-1.0",
-    "FreeImage",
-    "FSFAP",
-    "FSFUL",
-    "FSFULLR",
-    "FTL",
-    "GFDL-1.1-only",
-    "GFDL-1.1-or-later",
-    "GFDL-1.2-only",
-    "GFDL-1.2-or-later",
-    "GFDL-1.3-only",
-    "GFDL-1.3-or-later",
-    "Giftware",
-    "GL2PS",
-    "Glide",
-    "Glulxe",
-    "gnuplot",
-    "GPL-1.0-only",
-    "GPL-1.0-or-later",
-    "GPL-2.0-only",
-    "GPL-2.0-or-later",
-    "GPL-3.0-only",
-    "GPL-3.0-or-later",
-    "gSOAP-1.3b",
-    "HaskellReport",
-    "HPND",
-    "IBM-pibs",
-    "ICU",
-    "IJG",
-    "ImageMagick",
-    "iMatix",
-    "Imlib2",
-    "Info-ZIP",
-    "Intel-ACPI",
-    "Intel",
-    "Interbase-1.0",
-    "IPA",
-    "IPL-1.0",
-    "ISC",
-    "JasPer-2.0",
-    "JSON",
-    "LAL-1.2",
-    "LAL-1.3",
-    "Latex2e",
-    "Leptonica",
-    "LGPL-2.0-only",
-    "LGPL-2.0-or-later",
-    "LGPL-2.1-only",
-    "LGPL-2.1-or-later",
-    "LGPL-3.0-only",
-    "LGPL-3.0-or-later",
-    "LGPLLR",
-    "Libpng",
-    "libtiff",
-    "LiLiQ-P-1.1",
-    "LiLiQ-R-1.1",
-    "LiLiQ-Rplus-1.1",
-    "LPL-1.0",
-    "LPL-1.02",
-    "LPPL-1.0",
-    "LPPL-1.1",
-    "LPPL-1.2",
-    "LPPL-1.3a",
-    "LPPL-1.3c",
-    "MakeIndex",
-    "MirOS",
-    "MIT-advertising",
-    "MIT-CMU",
-    "MIT-enna",
-    "MIT-feh",
-    "MIT",
-    "MITNFA",
-    "Motosoto",
-    "mpich2",
-    "MPL-1.0",
-    "MPL-1.1",
-    "MPL-2.0-no-copyleft-exception",
-    "MPL-2.0",
-    "MS-PL",
-    "MS-RL",
-    "MTLL",
-    "Multics",
-    "Mup",
-    "NASA-1.3",
-    "Naumen",
-    "NBPL-1.0",
-    "NCSA",
-    "Net-SNMP",
-    "NetCDF",
-    "Newsletr",
-    "NGPL",
-    "NLOD-1.0",
-    "NLPL",
-    "Nokia",
-    "NOSL",
-    "Noweb",
-    "NPL-1.0",
-    "NPL-1.1",
-    "NPOSL-3.0",
-    "NRL",
-    "NTP",
-    "OCCT-PL",
-    "OCLC-2.0",
-    "ODbL-1.0",
-    "OFL-1.0",
-    "OFL-1.1",
-    "OGTSL",
-    "OLDAP-1.1",
-    "OLDAP-1.2",
-    "OLDAP-1.3",
-    "OLDAP-1.4",
-    "OLDAP-2.0.1",
-    "OLDAP-2.0",
-    "OLDAP-2.1",
-    "OLDAP-2.2.1",
-    "OLDAP-2.2.2",
-    "OLDAP-2.2",
-    "OLDAP-2.3",
-    "OLDAP-2.4",
-    "OLDAP-2.5",
-    "OLDAP-2.6",
-    "OLDAP-2.7",
-    "OLDAP-2.8",
-    "OML",
-    "OpenSSL",
-    "OPL-1.0",
-    "OSET-PL-2.1",
-    "OSL-1.0",
-    "OSL-1.1",
-    "OSL-2.0",
-    "OSL-2.1",
-    "OSL-3.0",
-    "PDDL-1.0",
-    "PHP-3.0",
-    "PHP-3.01",
-    "Plexus",
-    "PostgreSQL",
-    "psfrag",
-    "psutils",
-    "Python-2.0",
-    "Qhull",
-    "QPL-1.0",
-    "Rdisc",
-    "RHeCos-1.1",
-    "RPL-1.1",
-    "RPL-1.5",
-    "RPSL-1.0",
-    "RSA-MD",
-    "RSCPL",
-    "Ruby",
-    "SAX-PD",
-    "Saxpath",
-    "SCEA",
-    "Sendmail",
-    "SGI-B-1.0",
-    "SGI-B-1.1",
-    "SGI-B-2.0",
-    "SimPL-2.0",
-    "SISSL-1.2",
-    "SISSL",
-    "Sleepycat",
-    "SMLNJ",
-    "SMPPL",
-    "SNIA",
-    "Spencer-86",
-    "Spencer-94",
-    "Spencer-99",
-    "SPL-1.0",
-    "SugarCRM-1.1.3",
-    "SWL",
-    "TCL",
-    "TCP-wrappers",
-    "TMate",
-    "TORQUE-1.1",
-    "TOSL",
-    "Unicode-DFS-2015",
-    "Unicode-DFS-2016",
-    "Unicode-TOU",
-    "Unlicense",
-    "UPL-1.0",
-    "Vim",
-    "VOSTROM",
-    "VSL-1.0",
-    "W3C-19980720",
-    "W3C-20150513",
-    "W3C",
-    "Watcom-1.0",
-    "Wsuipa",
-    "WTFPL",
-    "X11",
-    "Xerox",
-    "XFree86-1.1",
-    "xinetd",
-    "Xnet",
-    "xpp",
-    "XSkat",
-    "YPL-1.0",
-    "YPL-1.1",
-    "Zed",
-    "Zend-2.0",
-    "Zimbra-1.3",
-    "Zimbra-1.4",
-    "zlib-acknowledgement",
-    "Zlib",
-    "ZPL-1.1",
-    "ZPL-2.0",
-    "ZPL-2.1",
+# A compiled, public domain list of official SPDX license tags.  generated
+# using: `python3 -m spdx_license_list print --filter-fsf-or-osi` Only contains
+# licenes approved by either FSF to be free/libre software or OSI to be open
+# source
+APPROVED_LICENSES = [
+    '0BSD',
+    'AAL',
+    'AFL-1.1',
+    'AFL-1.2',
+    'AFL-2.0',
+    'AFL-2.1',
+    'AFL-3.0',
+    'AGPL-3.0-only',
+    'AGPL-3.0-or-later',
+    'APL-1.0',
+    'APSL-1.0',
+    'APSL-1.1',
+    'APSL-1.2',
+    'APSL-2.0',
+    'Apache-1.0',
+    'Apache-1.1',
+    'Apache-2.0',
+    'Artistic-1.0',
+    'Artistic-1.0-Perl',
+    'Artistic-1.0-cl8',
+    'Artistic-2.0',
+    'Beerware',
+    'BSD-1-Clause',
+    'BSD-2-Clause',
+    'BSD-2-Clause-FreeBSD',
+    'BSD-2-Clause-Patent',
+    'BSD-3-Clause',
+    'BSD-3-Clause-Clear',
+    'BSD-3-Clause-LBNL',
+    'BSD-4-Clause',
+    'BSL-1.0',
+    'BitTorrent-1.1',
+    'CAL-1.0',
+    'CAL-1.0-Combined-Work-Exception',
+    'CATOSL-1.1',
+    'CC-BY-4.0',
+    'CC-BY-SA-4.0',
+    'CC0-1.0',
+    'CDDL-1.0',
+    'CECILL-2.0',
+    'CECILL-2.1',
+    'CECILL-B',
+    'CECILL-C',
+    'CNRI-Python',
+    'CPAL-1.0',
+    'CPL-1.0',
+    'CUA-OPL-1.0',
+    'ClArtistic',
+    'Condor-1.1',
+    'ECL-1.0',
+    'ECL-2.0',
+    'EFL-1.0',
+    'EFL-2.0',
+    'EPL-1.0',
+    'EPL-2.0',
+    'EUDatagrid',
+    'EUPL-1.1',
+    'EUPL-1.2',
+    'Entessa',
+    'FSFAP',
+    'FTL',
+    'Fair',
+    'Frameworx-1.0',
+    'GFDL-1.1-only',
+    'GFDL-1.1-or-later',
+    'GFDL-1.2-only',
+    'GFDL-1.2-or-later',
+    'GFDL-1.3-only',
+    'GFDL-1.3-or-later',
+    'GPL-2.0-only',
+    'GPL-2.0-or-later',
+    'GPL-3.0-only',
+    'GPL-3.0-or-later',
+    'HPND',
+    'IJG',
+    'IPA',
+    'IPL-1.0',
+    'ISC',
+    'Imlib2',
+    'Intel',
+    'LGPL-2.0-only',
+    'LGPL-2.0-or-later',
+    'LGPL-2.1-only',
+    'LGPL-2.1-or-later',
+    'LGPL-3.0-only',
+    'LGPL-3.0-or-later',
+    'LPL-1.0',
+    'LPL-1.02',
+    'LPPL-1.2',
+    'LPPL-1.3a',
+    'LPPL-1.3c',
+    'LiLiQ-P-1.1',
+    'LiLiQ-R-1.1',
+    'LiLiQ-Rplus-1.1',
+    'MIT',
+    'MIT-0',
+    'MIT-CMU',
+    'MPL-1.0',
+    'MPL-1.1',
+    'MPL-2.0',
+    'MPL-2.0-no-copyleft-exception',
+    'MS-PL',
+    'MS-RL',
+    'MirOS',
+    'Motosoto',
+    'MulanPSL-2.0',
+    'Multics',
+    'NASA-1.3',
+    'NCSA',
+    'NGPL',
+    'NOSL',
+    'NPL-1.0',
+    'NPL-1.1',
+    'NPOSL-3.0',
+    'NTP',
+    'Naumen',
+    'Nokia',
+    'OCLC-2.0',
+    'ODbL-1.0',
+    'OFL-1.0',
+    'OFL-1.1',
+    'OFL-1.1-RFN',
+    'OFL-1.1-no-RFN',
+    'OGTSL',
+    'OLDAP-2.3',
+    'OLDAP-2.7',
+    'OLDAP-2.8',
+    'OSET-PL-2.1',
+    'OSL-1.0',
+    'OSL-1.1',
+    'OSL-2.0',
+    'OSL-2.1',
+    'OSL-3.0',
+    'OpenSSL',
+    'PHP-3.0',
+    'PHP-3.01',
+    'PostgreSQL',
+    'Python-2.0',
+    'QPL-1.0',
+    'RPL-1.1',
+    'RPL-1.5',
+    'RPSL-1.0',
+    'RSCPL',
+    'Ruby',
+    'SGI-B-2.0',
+    'SISSL',
+    'SMLNJ',
+    'SPL-1.0',
+    'SimPL-2.0',
+    'Sleepycat',
+    'UCL-1.0',
+    'UPL-1.0',
+    'Unicode-DFS-2016',
+    'Unlicense',
+    'VSL-1.0',
+    'Vim',
+    'W3C',
+    'WTFPL',
+    'Watcom-1.0',
+    'X11',
+    'XFree86-1.1',
+    'Xnet',
+    'XSkat',
+    'YPL-1.1',
+    'ZPL-2.0',
+    'ZPL-2.1',
+    'Zend-2.0',
+    'Zimbra-1.3',
+    'Zlib',
+    'gnuplot',
+    'iMatix',
+    'xinetd',
 ]
 
+# an F-Droid addition, until we can enforce a better option
+APPROVED_LICENSES.append("PublicDomain")
+
 if __name__ == "__main__":
     main()
