@@ -41,17 +41,31 @@ def _run_wget(path, urls):
 def main():
     global options
 
-    parser = ArgumentParser(usage=_("%(prog)s [options] url"))
+    parser = ArgumentParser()
     common.setup_global_opts(parser)
     parser.add_argument("url", nargs='?',
                         help=_('Base URL to mirror, can include the index signing key '
                                + 'using the query string: ?fingerprint='))
+    parser.add_argument("--all", action='store_true', default=False,
+                        help=_("Mirror the full repo and archive, all file types."))
     parser.add_argument("--archive", action='store_true', default=False,
                         help=_("Also mirror the full archive section"))
+    parser.add_argument("--build-logs", action='store_true', default=False,
+                        help=_("Include the build logs in the mirror"))
+    parser.add_argument("--pgp-signatures", action='store_true', default=False,
+                        help=_("Include the PGP signature .asc files in the mirror"))
+    parser.add_argument("--src-tarballs", action='store_true', default=False,
+                        help=_("Include the source tarballs in the mirror"))
     parser.add_argument("--output-dir", default=None,
                         help=_("The directory to write the mirror to"))
     options = parser.parse_args()
 
+    if options.all:
+        options.archive = True
+        options.build_logs = True
+        options.pgp_signatures = True
+        options.src_tarballs = True
+
     if options.url is None:
         logging.error(_('A URL is required as an argument!') + '\n')
         parser.print_help()
@@ -73,7 +87,8 @@ def main():
 
         def _get_index(section, etag=None):
             url = _append_to_url_path(section)
-            return index.download_repo_index(url, etag=etag)
+            data, etag = index.download_repo_index(url, etag=etag)
+            return data, etag, _append_to_url_path(section, 'index-v1.jar')
     else:
         def _get_index(section, etag=None):
             import io
@@ -85,7 +100,7 @@ def main():
             with zipfile.ZipFile(io.BytesIO(content)) as zip:
                 jsoncontents = zip.open('index-v1.json').read()
             data = json.loads(jsoncontents.decode('utf-8'))
-            return data, etag
+            return data, etag, None  # no verified index file to return
 
     ip = None
     try:
@@ -124,18 +139,23 @@ def main():
     for section in sections:
         sectiondir = os.path.join(basedir, section)
 
-        data, etag = _get_index(section)
+        urls = []
+        data, etag, index_url = _get_index(section)
+        if index_url:
+            urls.append(index_url)
 
         os.makedirs(sectiondir, exist_ok=True)
         os.chdir(sectiondir)
         for icondir in icondirs:
             os.makedirs(os.path.join(sectiondir, icondir), exist_ok=True)
 
-        urls = []
         for packageName, packageList in data['packages'].items():
             for package in packageList:
                 to_fetch = []
-                for k in ('apkName', 'srcname'):
+                keys = ['apkName', ]
+                if options.src_tarballs:
+                    keys.append('srcname')
+                for k in keys:
                     if k in package:
                         to_fetch.append(package[k])
                     elif k == 'apkName':
@@ -145,7 +165,11 @@ def main():
                     if not os.path.exists(f) \
                        or (f.endswith('.apk') and os.path.getsize(f) != package['size']):
                         urls.append(_append_to_url_path(section, f))
-                        urls.append(_append_to_url_path(section, f + '.asc'))
+                        if options.pgp_signatures:
+                            urls.append(_append_to_url_path(section, f + '.asc'))
+                        if options.build_logs and f.endswith('.apk'):
+                            urls.append(_append_to_url_path(section, f[:-4] + '.log.gz'))
+
         _run_wget(sectiondir, urls)
 
         for app in data['apps']:
