@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 #
 # rewritemeta.py - part of the FDroid server tools
-# This cleans up the original .txt metadata file format.
+# This cleans up the original .yml metadata file format.
 # Copyright (C) 2010-12, Ciaran Gultnieks, ciaran@ciarang.com
 #
 # This program is free software: you can redistribute it and/or modify
@@ -21,6 +21,8 @@ from argparse import ArgumentParser
 import os
 import logging
 import io
+import tempfile
+import shutil
 
 from . import _
 from . import common
@@ -36,11 +38,8 @@ def proper_format(app):
     # read in metadata.py
     with open(app.metadatapath, 'r') as f:
         cur_content = f.read()
-    _ignored, extension = common.get_extension(app.metadatapath)
-    if extension == 'yml':
+    if app.metadatapath.endswith('.yml'):
         metadata.write_yaml(s, app)
-    elif extension == 'txt':
-        metadata.write_txt(s, app)
     content = s.getvalue()
     s.close()
     return content == cur_content
@@ -50,16 +49,11 @@ def main():
 
     global config, options
 
-    supported = ['txt', 'yml']
-
-    # Parse command line...
-    parser = ArgumentParser(usage="%(prog)s [options] [APPID [APPID ...]]")
+    parser = ArgumentParser()
     common.setup_global_opts(parser)
     parser.add_argument("-l", "--list", action="store_true", default=False,
                         help=_("List files that would be reformatted"))
-    parser.add_argument("-t", "--to", default=None,
-                        help=_("Rewrite to a specific format: ") + ', '.join(supported))
-    parser.add_argument("appid", nargs='*', help=_("applicationId in the form APPID"))
+    parser.add_argument("appid", nargs='*', help=_("application ID of file to operate on"))
     metadata.add_metadata_arguments(parser)
     options = parser.parse_args()
     metadata.warnings_action = options.W
@@ -67,30 +61,16 @@ def main():
     config = common.read_config(options)
 
     # Get all apps...
-    allapps = metadata.read_metadata(xref=True)
+    allapps = metadata.read_metadata(options.appid)
     apps = common.read_app_args(options.appid, allapps, False)
 
-    if options.list and options.to is not None:
-        parser.error(_("Cannot use --list and --to at the same time"))
-
-    if options.to is not None and options.to not in supported:
-        parser.error(_("Unsupported metadata format, use: --to [{supported}]")
-                     .format(supported=' '.join(supported)))
-
     for appid, app in apps.items():
         path = app.metadatapath
-        base, ext = common.get_extension(path)
-        if not options.to and ext not in supported:
-            logging.info(_("Ignoring {ext} file at '{path}'").format(ext=ext, path=path))
-            continue
-        elif options.to is not None:
-            logging.info(_("Rewriting '{appid}' to '{path}'").format(appid=appid, path=options.to))
-        else:
+        if path.endswith('.yml'):
             logging.info(_("Rewriting '{appid}'").format(appid=appid))
-
-        to_ext = ext
-        if options.to is not None:
-            to_ext = options.to
+        else:
+            logging.warning(_('Cannot rewrite "{path}"').format(path=path))
+            continue
 
         if options.list:
             if not proper_format(app):
@@ -98,7 +78,7 @@ def main():
             continue
 
         newbuilds = []
-        for build in app.builds:
+        for build in app.get('Builds', []):
             new = metadata.Build()
             for k in metadata.build_flags:
                 v = build[k]
@@ -106,12 +86,14 @@ def main():
                     continue
                 new[k] = v
             newbuilds.append(new)
-        app.builds = newbuilds
+        app['Builds'] = newbuilds
 
-        metadata.write_metadata(base + '.' + to_ext, app)
-
-        if ext != to_ext:
-            os.remove(path)
+        # rewrite to temporary file before overwriting existsing
+        # file in case there's a bug in write_metadata
+        with tempfile.TemporaryDirectory() as tmpdir:
+            tmp_path = os.path.join(tmpdir, os.path.basename(path))
+            metadata.write_metadata(tmp_path, app)
+            shutil.move(tmp_path, path)
 
     logging.debug(_("Finished"))
 
