@@ -64,13 +64,6 @@ class Decoder(json.JSONDecoder):
         return set(values), end
 
 
-class Encoder(json.JSONEncoder):
-    def default(self, obj):
-        if isinstance(obj, set):
-            return sorted(obj)
-        return super().default(obj)
-
-
 def write_json_report(url, remote_apk, unsigned_apk, compare_result):
     """write out the results of the verify run to JSON
 
@@ -118,7 +111,7 @@ def write_json_report(url, remote_apk, unsigned_apk, compare_result):
             data['packages'][packageName] = set()
         data['packages'][packageName].add(output)
         with open(jsonfile, 'w') as fp:
-            json.dump(data, fp, cls=Encoder, sort_keys=True)
+            json.dump(data, fp, cls=common.Encoder, sort_keys=True)
 
 
 def main():
@@ -128,7 +121,7 @@ def main():
     # Parse command line...
     parser = ArgumentParser(usage="%(prog)s [options] [APPID[:VERCODE] [APPID[:VERCODE] ...]]")
     common.setup_global_opts(parser)
-    parser.add_argument("appid", nargs='*', help=_("applicationId with optional versionCode in the form APPID[:VERCODE]"))
+    parser.add_argument("appid", nargs='*', help=_("application ID with optional versionCode in the form APPID[:VERCODE]"))
     parser.add_argument("--reuse-remote-apk", action="store_true", default=False,
                         help=_("Verify against locally cached copy rather than redownloading."))
     parser.add_argument("--output-json", action="store_true", default=False,
