@@ -33,7 +33,7 @@ import threading
 lock = threading.Lock()
 
 
-def get_clean_builder(serverdir, reset=False):
+def get_clean_builder(serverdir):
     if not os.path.isdir(serverdir):
         if os.path.islink(serverdir):
             os.unlink(serverdir)
@@ -51,26 +51,9 @@ def get_clean_builder(serverdir, reset=False):
                 end
                 """))
     vm = get_build_vm(serverdir)
-    if reset:
-        logging.info('resetting buildserver by request')
-    elif not vm.vagrant_uuid_okay():
-        logging.info('resetting buildserver, because vagrant vm is not okay.')
-        reset = True
-    elif not vm.snapshot_exists('fdroidclean'):
-        logging.info("resetting buildserver, because snapshot 'fdroidclean' is not present.")
-        reset = True
-
-    if reset:
-        vm.destroy()
-    vm.up()
-    vm.suspend()
-
-    if reset:
-        logging.info('buildserver recreated: taking a clean snapshot')
-        vm.snapshot_create('fdroidclean')
-    else:
-        logging.info('builserver ok: reverting to clean snapshot')
-        vm.snapshot_revert('fdroidclean')
+    logging.info('destroying buildserver before build')
+    vm.destroy()
+    logging.info('starting buildserver')
     vm.up()
 
     try:
@@ -117,11 +100,12 @@ def get_build_vm(srvdir, provider=None):
             logging.debug('build vm provider \'virtualbox\' selected')
             return VirtualboxBuildVm(abssrvdir)
         else:
-            logging.warn('build vm provider not supported: \'%s\'', provider)
+            logging.warning('build vm provider not supported: \'%s\'', provider)
 
     # try guessing provider from installed software
     kvm_installed = shutil.which('kvm') is not None
     kvm_installed |= shutil.which('qemu') is not None
+    kvm_installed |= shutil.which('qemu-kvm') is not None
     vbox_installed = shutil.which('VBoxHeadless') is not None
     if kvm_installed and vbox_installed:
         logging.debug('both kvm and vbox are installed.')
@@ -208,7 +192,22 @@ class FDroidBuildVm():
                 self.vgrnt.up(provision=provision, provider=self.provider)
                 self.srvuuid = self._vagrant_fetch_uuid()
             except subprocess.CalledProcessError as e:
-                raise FDroidBuildVmException("could not bring up vm '%s'" % self.srvname) from e
+                statusline = ""
+                try:
+                    # try to get some additional info about the vagrant vm
+                    status = self.vgrnt.status()
+                    if len(status) > 0:
+                        statusline = "VM status: name={n}, state={s}, provider={p}"\
+                            .format(n=status[0].name,
+                                    s=status[0].state,
+                                    p=status[0].provider)
+                except subprocess.CalledProcessError:
+                    pass
+                raise FDroidBuildVmException(value="could not bring up vm '{vmname}'"
+                                                   .format(vmname=self.srvname),
+                                             detail="{err}\n{statline}"
+                                                    .format(err=str(e), statline=statusline)
+                                             ) from e
 
     def suspend(self):
         global lock
@@ -429,13 +428,19 @@ class LibvirtBuildVm(FDroidBuildVm):
                   end""".format_map({'memory': str(int(domainInfo[1] / 1024)), 'cpus': str(domainInfo[3])}))
             with open('Vagrantfile', 'w') as fp:
                 fp.write(vagrantfile)
-            with tarfile.open(output, 'w:gz') as tar:
-                logging.debug('adding metadata.json to box %s ...', output)
-                tar.add('metadata.json')
-                logging.debug('adding Vagrantfile to box %s ...', output)
-                tar.add('Vagrantfile')
-                logging.debug('adding box.img to box %s ...', output)
-                tar.add('box.img')
+            try:
+                import libarchive
+                with libarchive.file_writer(output, 'gnutar', 'gzip') as tar:
+                    logging.debug('adding files to box %s ...', output)
+                    tar.add_files('metadata.json', 'Vagrantfile', 'box.img')
+            except (ImportError, AttributeError):
+                with tarfile.open(output, 'w:gz') as tar:
+                    logging.debug('adding metadata.json to box %s ...', output)
+                    tar.add('metadata.json')
+                    logging.debug('adding Vagrantfile to box %s ...', output)
+                    tar.add('Vagrantfile')
+                    logging.debug('adding box.img to box %s ...', output)
+                    tar.add('box.img')
 
             if not keep_box_file:
                 logging.debug('box packaging complete, removing temporary files.')
@@ -444,8 +449,8 @@ class LibvirtBuildVm(FDroidBuildVm):
                 os.remove('box.img')
 
         else:
-            logging.warn("could not connect to storage-pool 'default', "
-                         "skip packaging buildserver box")
+            logging.warning("could not connect to storage-pool 'default', "
+                            "skip packaging buildserver box")
 
     def box_add(self, boxname, boxfile, force=True):
         boximg = '%s_vagrant_box_image_0.img' % (boxname)
